-- Vector class definition File
-- Requires nothing


-- Vector Class
Vector2d = {x=0,y=0}


-- Vector Class Constructor
function Vector2d:new(x,y)
	local v = {x = x or 0, y = y or 0}
	self.__index = self
	return setmetatable(v,self)
end

-- Tests if a set of vector are equals (component by component)
function Vector2d.equals(...)
	for i,v1 in ipairs(arg) do
		for j,v2 in ipairs(arg) do
			if i~=j then
				if v1.x~=v2.x or v1.y ~= v2.y then
				return false
				end
			end
		end
	end
	return true
end

-- Adds two vectors
-- Overrides + operator
function Vector2d.__add(v,a)
	local res = {}
	res.x = (v.x or v) + (a.x or a)
	res.y = (v.y or v) + (a.y or a)
	return Vector2d:new(unpack(res))
end

-- Substracts two vectors
-- Overrides - operator
function Vector2d.__sub(v,a)
	local res = {}
	res.x = (v.x or v) - (a.x or a)
	res.y = (v.y or v) - (a.y or a)
	return Vector2d:new(unpack(res))
end

-- Adds a vector to self
function Vector2d:add(v)
	self.x = self.x + (v.x or 0)
	self.y = self.y + (v.y or 0)
end

-- Substract a vector from self
function Vector2d:sub(v)
	self.x = self.x - (v.x or 0)
	self.y = self.y - (v.y or 0)
end

-- Scales a vector by a real number
function Vector2d:scale(a)
	self.x = self.x*a
	self.y = self.y*a
end

-- Adds a scaled vector v by a real number a to self
function Vector2d:addScaledVector(v,a)
	self.x = self.x + (v.x*a)
	self.y = self.y + (v.y*a)
end

-- Scales a vector by -1
function Vector2d:invert()
	self:scale(-1)
end

-- Computes self magnitude
function Vector2d:magnitude()
	return math.sqrt((self.x^2+self.y^2))
end

-- Computes self squared magnitude
function Vector2d:squareMagnitude()
	return (self.x^2+self.y^2)
end

-- Normalizes self
function Vector2d:normalize()
	local l = self:magnitude()
	self.x = (l>0) and self.x/l or self.x
	self.y = (l>0) and self.y/l or self.y
end

-- Return unit vector from self
function Vector2d:unit()
	if self:magnitude() > 0 then
		local v = self
		v:normalize()
		return v
	else
		return self
	end
end

-- Component by component product
-- Overrides * operator
function Vector2d.__mul(v,a)
	local res = {}
	res.x = (v.x or v) * (a.x or a)
	res.y = (v.y or v) * (a.y or a)
	return Vector2d:new(unpack(res))
end

-- Computes scalar product
function Vector2d:scalarProduct(v)
	return (self.x*v.x+self.y*v.y)
end

-- Limits self component to size
function Vector2d:trim(size)
	if self:squareMagnitude() > size*size then
	self.x = size
	self.y = size
	end
end

-- Tests equality between two vectors
-- Overrides = operator
function Vector2d.__eq(v1,v2)
	return (v1.x==v2.x) and (v1.y == v2.y)
end

-- Tests if self is less than vector v (component by component)
function Vector2d:lessThan(v)
	return (self.x < v.x) and (self.y < v.y)
end

-- Tests if self is less or equal to vector v (component by component)
function Vector2d:lessOrEqualThan(v)
	return (self.x <= v.x) and (self.y <= v.y)
end

-- Tests if self is higher than vector v (component by component)
function Vector2d:highThan(v)
	return (self.x > v.x) and (self.y > v.y)
end

-- Tests if self is less higher or equal to vector v (component by component)
function Vector2d:highOrEqualThan(v)
	return (self.x >= v.x) and (self.y >= v.y)
end

-- Resets self to Zero-Vector
function Vector2d:clear()
	self.x,self.y = 0,0
end

-- Prints self components
-- For debugging purposes
function Vector2d:print(id)
	print((string.gsub(" id : x  = "..self.x.."  y = "..self.y,"id",id or "vector")))
end


