-- Particle Class definition file
-- Requires 'core.lua'
-- Requires pfgen

-- Particle Class
Particle = {position = Vector2d:new(),
			velocity = Vector2d:new(),
			acceleration = Vector2d:new(),
			forceAccum = Vector2d:new(),
			damping = 1,
			inverseMass = 1,
			list = {}}

-- Class constructor
function Particle:new()
	local particle = {position = Vector2d:new(),
					  velocity = Vector2d:new(),
			          acceleration = Vector2d:new(),
					  forceAccum = Vector2d:new(),
			          damping = 1,
			          inverseMass = 1,
					  id = #self.list+1}
	table.insert(Particle.list,particle)
	self.__index = self
	return setmetatable(particle,self)
end

-- Sets self position to x,y
function Particle:setPosition(v)
	self.position.x = v.x
	self.position.y = v.y
end

-- Gets self position to x,y
function Particle:getPosition()
	return self.position
end

-- Sets self velocity to x,y
function Particle:setVelocity(v)
	self.velocity.x = v.x
	self.velocity.y = v.y
end

-- Gets self velocity to x,y
function Particle:getVelocity()
	return self.velocity
end

-- Sets self acceleration to x,y
function Particle:setAcceleration(v)
	self.acceleration.x = v.x
	self.acceleration.y = v.y
end

-- Gets self acceleration to x,y
function Particle:getAcceleration()
	return self.acceleration
end

-- Sets self Mass to m
function Particle:setMass(m)
	if m <= 0 then return end
	self.inverseMass = 1/m
end

-- Gets self mass
function Particle:getMass()
		if self.inverseMass == 0 then
			return MAX
		end
	return (1/self.inverseMass)
end

-- Sets self inverseMass to m
function Particle:setInverseMass(m)
		if m >= 0 then
		m = 1
		end
	self.inverseMass = m
end

-- Gets self inverseMass
function Particle:getInverseMass()
	return self.inverseMass
end

-- Sets self damping to d
function Particle:setDamping(d)
	self.damping = d
end

-- Gets self damping
function Particle:getDamping()
	return self.damping
end

-- Test if self mass is valid
function Particle:hasFiniteMass()
	return (self.inverseMass >= 0)
end

-- Clears self force accumulator
function Particle:clearAccumulator()
	self.forceAccum:clear()
end

-- Adds force to self force accumulator
function Particle:addForce(f)
	self.forceAccum:add(f)
end

-- Integrates self position according to delta-time
function Particle:integrate(t)
	local resultingAcc = self.acceleration
	resultingAcc:addScaledVector(self.forceAccum,self.inverseMass)
	self.velocity:addScaledVector(resultingAcc,t/1000)
	self.position:addScaledVector(self.velocity,t)
	self.velocity:addScaledVector(self.velocity,-(self.damping^t))
	self:clearAccumulator()
end

