-- Particle Physics Demo 
-- Uses 'PhysicsEngine'
-- roland.yonaba[at]gmail[dot]com

require 'Engine/core'
require 'Engine/pfgen'
require 'Engine/particle'

-- This is not related to the Engine. 
-- Consider this as a simple table where we stores 
-- the current example paramaters : 
-- Damping, Window Width, Window height
local world = {damping = 1E-2,
			   width = love.graphics.getWidth(),
			   height = love.graphics.getHeight(),			   
			   }

-- We creates a shooter with three types of weapons			   
local shooter = {types = {"LASER","BULLET","ARTILLERY"},
				 ammoIm = {["LASER"] = love.graphics.newImage("Img/laser.png"),
						   ["BULLET"] = love.graphics.newImage("Img/bullet.png"),
						   ["ARTILLERY"] = love.graphics.newImage("Img/ball.png"),
						   },							
				 location = {x = 10, y = love.graphics.getHeight()/2},				
				 currentType = "LASER",
				 Ammos = {},
				 AmmoPatterns = {["LASER"]= {mass = 1,velocity = {x = 0.6,y = 0},acceleration = {x = 0.3,y = 0}},
								["BULLET"]= {mass = 2,velocity = {x = 0.5,y = 0},acceleration = {x = 0.1,y = 0}},
								["ARTILLERY"]= {mass = 5,velocity = {x = 0.3,y = -0.1},acceleration = {x = 0.05,y = 0.05}},								
								},
				 hasFired = false,
				 count = 0,
				}
				
-- Sets the actual type of shooter for the next
-- shot according to the keyboard input.				
function shooter.setType(key)
	local cur,wp
	cur = tonumber(key)
		if not cur then return end
	wp = shooter.types[cur]
		if not wp then return end
	shooter.currentType = shooter.types[cur]
end

-- Checks if a fired ammo is still in the 
-- current window boundaries.If no, ammo is cleared from memory.
function shooter.checkAmmos()
	for _,Ammo in ipairs(shooter.Ammos) do
		local outOfBouds = ((Ammo.position.x > world.width+100) or (Ammo.position.y > world.height+100) or
						(Ammo.position.x < -100) or (Ammo.position.y < -100))
		if outOfBouds then
		table.remove(shooter.Ammos,_)
		end
	end
end

-- Creates and stores a new Ammo 						
function shooter.createAmmo()
	local Ammo = Particle:new()
	Ammo:setMass(shooter.AmmoPatterns[shooter.currentType].mass)
	Ammo:setVelocity(shooter.AmmoPatterns[shooter.currentType].velocity)
	Ammo:setAcceleration(shooter.AmmoPatterns[shooter.currentType].acceleration)
	Ammo:setPosition(shooter.location)
	Ammo:setDamping(world.damping)
	Ammo:clearAccumulator()
	Ammo.im = shooter.ammoIm[shooter.currentType]
	GRAVITY:connect(Ammo)
	table.insert(shooter.Ammos,Ammo)
	shooter.count = shooter.count+1
end

-- Manages Ammos in the roster, listens for mouse input
function shooter.orderAmmo()
	shooter.checkAmmos()
	if love.mouse.isDown('l','r') then
		if not shooter.hasFired then		
		shooter.hasFired = true
		shooter.createAmmo()
		end
	else
	shooter.hasFired = false
	end		
end


-- Updates Ammos in the roster position through time (Callback)
function love.update(dt)
	--world.forces:updateForces(dt*(1000/3))
	pfgen:update(dt*(1000/3))
	for _,Ammo in ipairs(shooter.Ammos) do
	Ammo:integrate(dt*(1000/3))
	end
end

-- Drawing Scene (Callback)
love.graphics.setFont(8)
function love.draw()
	love.graphics.print('Current Weapon : '..shooter.currentType,10,30)
		if shooter.Ammos[#shooter.Ammos] then love.graphics.print('Last Ammo fired Mass : '..(shooter.Ammos[#shooter.Ammos]:getMass() or ""),10,50) end
	love.graphics.print('Ammos in the roster : '..#shooter.Ammos,10,70)
	love.graphics.print('Ammos Fired : '..shooter.count,10,90)
	love.graphics.print('Click Mouse-left/Mouse-Right to Fire',10,110)
	love.graphics.print('Keyboard 1-3 : Toggle weapons',10,130)
	
	shooter.orderAmmo()
	for _,Ammo in ipairs(shooter.Ammos) do
	love.graphics.draw(Ammo.im,Ammo.position.x,Ammo.position.y)
	end
	
	if #shooter.Ammos > 0 then
	love.graphics.print(" Last Ammo : x            : "..shooter.Ammos[#shooter.Ammos].position.x.."  y : "..shooter.Ammos[#shooter.Ammos].position.y,10,10)
	end

end

-- Keyboard input (Callback)
function love.keypressed(key,unicode)
	shooter.setType(key)
end
